<?php
namespace Teknisa\PED\Modules\Expedicao\Controllers;

use Teknisa\PED\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\ParameterBag;
use \Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use \Doctrine\ORM\EntityManager;
use Teknisa\PED\Services\ReservaPedidoService;
use Teknisa\PED\Services\ValidacoesEstoqueService;
use \Teknisa\Libs\Util\DateUtil;


use Teknisa\PED\Modules\Expedicao\Services\Exp00200LancamentoExpedicaoPedidoService as Exp00200LancamentoExpedicaoPedidoService;

class Exp00200LancamentoExpedicaoPedidoController extends TeknisaCrud {

    protected $dataSourceName = '';
    protected $validacoesEstoqueService;
    protected $reservaPedidoService;
    protected $ExpedicaoService;
    protected $newCode;
    protected $environment;
    protected $entityManager;

    public function __construct($crudService, ParameterBag $parameterBag, Exp00200LancamentoExpedicaoPedidoService $exp00200LancamentoExpedicaoPedidoService,
    ReservaPedidoService $reservaPedidoService, ValidacoesEstoqueService $validacoesEstoqueService, $newCode, $environment, $entityManager) {
        parent::__construct($crudService, $parameterBag);
        $this->ExpedicaoService = $exp00200LancamentoExpedicaoPedidoService;
        $this->reservaPedidoService = $reservaPedidoService;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }

    public function getDataProc($cdfilial){
        return $this->ExpedicaoService->getDataProc($cdfilial);
    }
    
    public function insereNrExpedicao($pedido,$paramGeraCorpoNf, $nrexpedicao){
       
        $cdfilial = $pedido['CDFILIAL'];
        $cdfilialped = $pedido['CDFILIALPED'];
        $cdviatura = $pedido['CDVIATURA'];
        $dsqtdevolped = $pedido['DSQTDEVOLPED'];
        $cdopergera = $pedido['CDOPERADOR'];
        $nrlanctonf = $paramGeraCorpoNf['P_NRLANCTONF'];
        $cdopersep = isset($pedido['CDOPERSEP']) ? $pedido['CDOPERSEP'] : null;
        $cdoperconf = isset($pedido['CDOPERCONF'])? $pedido['CDOPERCONF'] :null;
        $cdrotatrans = isset($pedido['CDROTATRANS']) ? $pedido['CDROTATRANS'] : null;
        $this->ExpedicaoService->insereNrExpedicao($cdfilial, $nrexpedicao, $cdfilialped, $cdviatura, $cdopersep, $cdoperconf);
        $this->ExpedicaoService->updateNrExpedicao($cdfilial, $nrexpedicao, $cdrotatrans, $dsqtdevolped, $nrlanctonf, $cdopergera);
        
    }

    public function expedirPedido($request, $response){

        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        $pedidos = $rows[0];
        $lotesExp = $rows[1];
        $this->entityManager->getConnection()->beginTransaction();
        $retorno = array();
        try {
            foreach ($pedidos as $pedido) {
                // Verificar se posso expedir o pedido:
                // - Se tem de onde tirar  esse pedido (Qual almoxarifado, qual)
                // - Se tem os itens  no estoque para que eu possa expedir
                $nrpedido = $pedido['NRPEDIDO'];
                $cdfilial = $pedido['CDFILIAL'];
                $organizacao = $pedido['NRORG'];
                $sysdate = DateUtil::getDataAtual()->format('d/m/Y');
                if (!(isset($pedido['CDSERIEENTR']))) {
                    $pedido['CDSERIEENTR'] = null;
                }
                
                $isValid = $this->ExpedicaoService->validaExpedicao($pedido);
                
                
                //dentro dessa função tem varias validações, virificar quais devem continuar
                $itensPedido = $this->ExpedicaoService->getItensPed($cdfilial, $nrpedido);
                $itensPedido = $this->verificaMoeda($nrpedido, $cdfilial, $itensPedido);
                $lotesPedido = array();
                for ($i=0; $i < count($lotesExp); $i++) {
                    if ($nrpedido == $lotesExp[$i]['NRPEDIDO']) {
                        array_push($lotesPedido, $lotesExp[$i]);
                    }
                }
                $paramsFilial = $this->ExpedicaoService->getParamsFilial($cdfilial);
                $dtProcessa = $this->validacoesEstoqueService->retornaDadosProcessFilial($cdfilial, $organizacao);
                $dtProcessa = $dtProcessa['DTPROCESSA'];
                $dtProcessa = DateUtil::getDataDeString($dtProcessa, "d/m/Y H:i:s")->format('d/m/Y');
                
                $tipooper    = $this->ExpedicaoService->getTipooperac($pedido['CDTIPOOPER']);
                $dadosFilial = $this->validaDadosFilial($cdfilial, $organizacao);
                
                // NOVA VALIDAÇÃO DE ESTOQUE
                // Verifica se filial controla estoque
                if($paramsFilial && $paramsFilial[0]['IDCTRLESTQ'] == 'S' &&
                    // Verifica se tipo de operação controla estoque
                    $tipooper && $tipooper[0]['IDATUESTTPOP'] == 'S' &&
                    // Verifica se o filial controla almoxarifado
                    $dadosFilial && $dadosFilial[0]['IDUTILALMOX'] == 'S'){
                        
                    // Verifica se o Item tem almoxarifado cadastrado, caso não tenha lança um erro;
                    $itensPedido = $this->almoxarifadoLocalEstqItens($itensPedido);
                    if (isset($itensPedido['DSERRO'])) {
                        array_push($retorno, $itensPedido);
                    } else {
                        // Verifica se o Item possui quantidade sufuciente em estoque;
                        $itensPedido = $this->qtEstoqueItens($itensPedido, $dadosFilial, $cdfilial, $lotesPedido);
                        if (isset($itensPedido['DSERRO'])) {
                            array_push($retorno, $itensPedido);
                        }
                    }
                            
                }else{
                    foreach ($itensPedido as $key => $item) {
                        $itensPedido[$key]['CDALMOXARIFE'] = ' ';
                        $itensPedido[$key]['CDLOCALESTOQ'] = ' ';
                    }
                }
                // FIM NOVA VALIDAÇÃO DE ESTOQUE
                
                if (count($isValid) > 0 || isset($itensPedido['DSERRO'])) {
                    $retorno = array_merge($retorno, $isValid);
                    continue;
                    // throw new \Exception($itensPedido['DSERRO']);
                    
                }
                if(isset($pedido['CDTIPOOPER'])){
                    $paramGeraCorpoNf = $this->geraCorpoNf($pedido, $cdfilial, $sysdate, $paramsFilial, $dtProcessa);
                    
                    /** Chamada da procedure de GERA_NOTAFISCAL_CORPO **/
                    try {
                        
                        $this->entityManager->getConnection()->beginTransaction();
                        
                        $retornoCorpo = $this->callProcedureGERA_NOTAFISCAL_CORPO($paramGeraCorpoNf);
                        $this->entityManager->commit();
                    } catch(\Exception $e) {
                        $this->entityManager->rollback();
                        throw $e;
                    }
                    
                    /** Loop de Itens e chamada da procedure de GERA_NOTAFISCAL_PRODUTO  */
                    try {
                        $this->entityManager->getConnection()->beginTransaction();
                        foreach ($itensPedido as $item) {
                            $lotesItem = array();
                            for ($i=0; $i < count($lotesPedido); $i++) {
                                if ($item['NRPEDIDO'] == $lotesPedido[$i]['NRPEDIDO']) {
                                    array_push($lotesItem, $lotesPedido[$i]);
                                }
                            }
                            $itemNf = $this->geraItemNf($pedido, $item, $dadosFilial, $cdfilial, $nrpedido, $sysdate, $paramGeraCorpoNf['P_NRLANCTONF'], $lotesItem);
                         
                            for ($i = 0; $i < count($itemNf); $i++) {
                                $retornoProduto = $this->callProcedureGERA_NOTAFISCAL_PRODUTO($itemNf[$i]);
                                
                                if (!(isset($retornoProduto)) && empty($retornoProduto) && count($retornoProduto) > 0) {
                                    throw new \Exception($retornoProduto);
                                }
                            
                            }
                        }
                            
                        $this->entityManager->commit();
                    } catch(\Exception $e){
                        $this->entityManager->rollback();
                        throw $e;
                    }
                    
                    /** Loop de Encargo e chamada da procedure de GERA_NOTAFISCAL_ENCARGO */
                    try {
                        $this->entityManager->getConnection()->beginTransaction();
                        $encargoPed = $this->ExpedicaoService->getEncargoPed($cdfilial, $nrpedido);
                        foreach ($encargoPed as $encargo) {
                            if(count($encargo) > 0) {
                                $encargoNF = array(
                                    'P_CDFILIAL'    => $cdfilial,
                                    'P_NRLANCTONF'  => $paramGeraCorpoNf['P_NRLANCTONF'],
                                    'P_CDENCARGO'   => $encargo['CDENCARGO'],
                                    'P_VRITEMENCA'  => $encargo['VALOR']
                                );
                                $retornoEncargo = $this->callProcedureGERA_NOTAFISCAL_ENCARGO($encargoNF);
                                if(count($retornoEncargo) > 0) {
                                    throw new \Exception($retornoEncargo);
                                }
                            }
                        }
                        $this->entityManager->commit();
                    } catch(\Exception $e){
                        $this->entityManager->rollback();
                        throw $e;
                    }
                        
                    /** Loop de Dados Adicionais e chamada da procedure de GERA_NOTAFISCAL_DADOSADIC **/
                    try {
                        $this->entityManager->getConnection()->beginTransaction();
                        $dadosAdicPed = $this->ExpedicaoService->getDadosAdicPed($cdfilial, $nrpedido);
                        foreach ($dadosAdicPed as $dados) {
                            $dadosadicNF = array(
                                'P_CDFILIAL' => $cdfilial,
                                'P_NRLANCTONF' => $paramGeraCorpoNf['P_NRLANCTONF'],
                                'P_DTINICIO' => $dtProcessa,
                                'P_DTFINAL' => $dtProcessa,
                                'P_MENSAGEM' => $dados['DSDADOADICPED']
                            );
                            $retornoDadosAdicNF = $this->callProcedureGERA_NOTAFISCAL_DADOSADIC($dadosadicNF);
                            if(count($retornoDadosAdicNF) > 0) {
                                throw new \Exception($retornoDadosAdicNF);
                            }
                        }
                        $this->entityManager->commit();
                    } catch(\Exception $e){
                        $this->entityManager->rollback();
                        throw $e;
                    }
                    
                    /** Chamada da procedure de GERA_NOTAFISCAL_FECHAMENTO **/
                    try {
                        $this->entityManager->getConnection()->beginTransaction();
                        $fechamentoNF = array(
                            'P_CDFILIAL'    => $cdfilial,
                            'P_NRLANCTONF'  => $paramGeraCorpoNf['P_NRLANCTONF'],
                            'P_NRLANCESTQ'  => $paramGeraCorpoNf['P_NRLANCESTQ'],
                            'P_DATAPROC'    => $dtProcessa,
                            'P_EVOLESTOQ'   => 'S',
                            'P_IDBLOQTRANS' => 'N',
                            'P_IDATUESTOQ'  => 'S'
                        );
                        $retornoFechamentoNF = $this->callProcedureGERA_NOTAFISCAL_FECHAMENTO($fechamentoNF);
                        
                        if(gettype($retornoFechamentoNF) != "array"){
                            throw new \Exception($retornoFechamentoNF);
                        }
                        $this->entityManager->commit();
                    } catch(\Exception $e){
                        $this->entityManager->rollback();
                        throw $e;
                    }
                    
                    try{
                        $this->entityManager->getConnection()->beginTransaction();
                        $nrexpedicao = $this->getNewCode('NREXPEDICAO', $cdfilial);
                        $this->insereNrExpedicao($pedido, $paramGeraCorpoNf, $nrexpedicao);
                        $this->entityManager->commit();
                    } catch(\Exception $e){
                        $this->entityManager->rollback();
                        throw $e;
                    }
    
                    
                    try {
                        $this->entityManager->getConnection()->beginTransaction();
                        $this->dataSourceNameToSave = 'pedido';
                        $pedido['IDSITPEDIDO'] = '3';
                        $requestSalvar = new DTO\Request\Row($pedido, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                        parent::save($requestSalvar, $response);
    
                        
                        $this->entityManager->commit();
                    } catch(\Exception $e){
                        $this->entityManager->rollback();
                        throw $e;
                    }
                    
                    try {
                        $this->entityManager->getConnection()->beginTransaction();
                            
                        $this->dataSourceNameToSave = 'itempedido';
                        
                        foreach ($itensPedido as &$item) {
                            $item['IDSITITEMPED'] = '4';
                            $item['__is_new'] = false;
                        }
                        
                        $dataSetItem = new DataSet ("data", $itensPedido);
                        $requestSalvar = new DTO\Request\DataSet($dataSetItem, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                        parent::save($requestSalvar, $response);
                        $this->entityManager->commit();
                    } catch(\Exception $e){
                        $this->entityManager->rollback();
                        throw $e;
                    }
                    array_push($retorno, array("CDFILIAL"=> $pedido['CDFILIAL'],"NRPEDIDO"=> $pedido['NRPEDIDO'], "NRLANCTONF" => $paramGeraCorpoNf['P_NRLANCTONF'], "DSERRO"=> "Expedido com Sucesso", "NREXPEDICAO"=> $nrexpedicao));
                }else{
                    array_push($retorno, array("CDFILIAL"=> $pedido['CDFILIAL'],"NRPEDIDO"=> $pedido['NRPEDIDO'], "DSERRO"=> "Impossível expedir o pedido. Pedido sem Tipo de Operação cadastrado."));
                }
            }
            //  tratar a variavel retorno e se retornar para o frontend;
            $returno = new DataSet ("data", $retorno);
            $response->addDataSet($returno);
            $this->entityManager->commit();
        } catch (\Exception $e){
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function verificaMoeda($nrpedido, $cdfilial, $itensPedido){
        $sysdate = DateUtil::getDataAtual();
        $data = DateUtil::subtraiIntervalo($sysdate, 1)->format('d/m/Y'); // TROCAR PARA 1
        
        $indiceFinanceiro = $this->ExpedicaoService->verificaIndice($nrpedido, $cdfilial, $data);
        if (isset($indiceFinanceiro) && count($indiceFinanceiro) > 0) {
            foreach ($itensPedido as &$item) {
                $item['VRDESITPEDCONVERT'] = $item['VRDESITPED'] * $indiceFinanceiro[0]['VRINDICE'];
                $item['VRITEMPEDCONVERT'] = $item['VRPREUNITPED'] * $indiceFinanceiro[0]['VRINDICE'];
            }
        } else {
            foreach ($itensPedido as &$item) {
                $item['VRDESITPEDCONVERT'] = $item['VRDESITPED'];
                $item['VRITEMPEDCONVERT'] = $item['VRPREUNITPED'];
            }
        }
        return $itensPedido;
    }
   
    //Retorna se a filial controla almoxarifado, localestoque e lote
    private function validaDadosFilial($cdfilial, $organizacao){
        $dados = $this->reservaPedidoService->retornaDadosFilial($cdfilial, $organizacao);
        return $dados;
    }

    private function almoxarifadoLocalEstqItens($itensPedido){
        $mensage = "";
        $erros = array();
        $dadosAlmoxarif = array();
        foreach ($itensPedido as $key => $item) {
            
            // o parametro deve ser o CDPRODESTO para pegar  oproduto de estoque nao o de compra
            $dadosAlmoxarif[$key] = $this->reservaPedidoService->verificaAlmoxarifadoLoteProduto($item['CDFILIAL'], $item['CDPRODESTO']);
            
            if (!($dadosAlmoxarif[$key])) {
                array_push($erros, $item);
            } else {
                $dadosAlmoxarif[$key] = $this->reservaPedidoService->retornaAlmoxarifadoLoteProduto($item['CDFILIAL'], $item['CDPRODESTO']); 
                $itensPedido[$key]['CDALMOXARIFE'] = $dadosAlmoxarif[$key][0]['CDALMOXPRCLI'];
                $itensPedido[$key]['CDLOCALESTOQ'] = $dadosAlmoxarif[$key][0]['CDLOCALPRCLI'];
            }
        }
        if (count($erros) == 1) {
            $mensage = " não possui parametrização na tela de Local de Estoque por Produto.";
            return  array("CDFILIAL"=> $item['CDFILIAL'], "NRPEDIDO"=> $item['NRPEDIDO'], "DSERRO"=> "$mensage", "PRODUTO"=> $erros[0]['NMPRODUTO']);
        } else if (count($erros) > 1) {
            $produtos = "";
            foreach ($erros as $key => $item) {
                $produtos = $produtos.$item['NMPRODUTO'];
                if ($key == count($erros)) {
                    $produtos = $produtos.' ';
                } else {
                	$produtos = $produtos.', ';
                }
            }
            $mensage = 'não possuem parametrização na tela de Local de Estoque por Produto.';
            return array("CDFILIAL"=> $item['CDFILIAL'], "NRPEDIDO"=> $item['NRPEDIDO'], "DSERRO"=> "$mensage", "PRODUTO"=> $produtos);
        } else {
            return $itensPedido;
        }
    }

    private function qtEstoqueItens($itensPedido, $dadosFilial, $cdfilial, $lotesPedido = 0){
        $mensage = "";
        $erros = array();
        $infQtitens = array();

        //Caso o lote seja definido no Frontend pula estas validações e utiliza as selecionadas pelo usuario
        // if (count($lotesPedido) <= 0) {
            foreach ($itensPedido as $key => $item) {
                $error = false;
                $qtitens = 0;
                if ($dadosFilial[0]['IDUTILALMOX'] == 'S'){
                    $infQtitens[$key] = $this->reservaPedidoService->buscaEstoqueAlmox($cdfilial, $item['NRPEDIDO'], $item['NRITEMPED'], $item['CDPRODESTO'], $item['CDALMOXARIFE'], $item['CDLOCALESTOQ']);
                } else {
                    $infQtitens[$key] = $this->reservaPedidoService->buscaEstoque($cdfilial, $item['NRPEDIDO'], $item['NRITEMPED'], $item['CDPRODESTO'], $item['CDALMOXARIFE'], $item['CDLOCALESTOQ']);
                }
                if (!($infQtitens[$key])) {
                    $error = true;
                    array_push($erros, $item);
                } else {
                    $itensPedido[$key]['CDALMOXARIFE'] = $infQtitens[$key][0]['CDALMOXARIFE'];
                    $itensPedido[$key]['CDLOCALESTOQ'] = $infQtitens[$key][0]['CDLOCALESTOQ'];
                }
                for ($i = 0; $i < count($infQtitens[$key]); $i++ ) {
                    $qtitens += $infQtitens[$key][$i]['QTDISPONIVEL'];
                }
                if ($item['QTITEMPED'] > $qtitens && !($error)) {
                    array_push($erros, $item);
                }
            // }
        }

        if (count($erros) == 1) {
            $mensage = " não possui quantidade suficiente em estoque.";
            return  array("CDFILIAL"=> $item['CDFILIAL'], "NRPEDIDO"=> $item['NRPEDIDO'], "DSERRO"=> "$mensage", "PRODUTO"=> $erros[0]['NMPRODUTO']);
        } else if (count($erros) > 1) {
            $produtos = "";
            foreach ($erros as $key => $item) {
                $produtos = $produtos.$item['NMPRODUTO'];
                if ($key == count($erros)) {
                    $mensage = $mensage.' ';
                } else {
                	$mensage = $mensage.', ';
                }
            }
            $mensage = "não possuem quantidade suficiente em estoque.";
            return  array("CDFILIAL"=> $item['CDFILIAL'], "NRPEDIDO"=> $item['NRPEDIDO'], "DSERRO"=> "$mensage", "PRODUTO"=> $produtos);
        } else {
            return $itensPedido;
        }
    }

    public function getEstoqueItemData($request, $response){
        $row = $request->getRow();
        $data = $this->reservaPedidoService->verificaEstProd($row['CDFILIAL'], $row['NRPEDIDO'], $row['NRITEMPED'], $row['CDPRODESTO'], $row['CDALMOXARIFE'], $row['CDLOCALESTOQ']);
        $dataSet = new DataSet ("data", $data);
        $response->addDataSet($dataSet);
    }

    public function geraCorpoNf($pedido, $cdfilial, $sysdate, $paramsFilial, $dtProcessa){

        $tipooper = $this->ExpedicaoService->getTipooperac($pedido['CDTIPOOPER']);
        $paramsExp = $this->ExpedicaoService->getParamsExp($pedido['CDFILIAL']);
        $viatura = $this->ExpedicaoService->getViatura($pedido['CDVIATURA']);
        $tranportadora = $this->ExpedicaoService->getTransportadora($pedido['NRINSJURTRAN']);
        
        if ($tipooper && $tipooper[0]['IDORIGDESTOP'] == 'C'){ //Cliente
            $p_cdcliente = $pedido['CDCLIENTE'];
        } else if ($tipooper[0]['IDORIGDESTOP'] == 'F') { // Fornecedor
            $p_cdcliente = $pedido['CDFORNECEDPED'];
        } else if ($tipooper[0]['IDORIGDESTOP'] == 'L') { // Filial
            $p_cdcliente = $pedido['CDFILIALPED'];
        }
        
        if ($tipooper && $tipooper[0]['IDATUESTTPOP'] == 'S' && $paramsFilial[0]['IDCTRLESTQ'] == 'S'){
            $p_atuaestoque = 'S';
        } else {
            $p_atuaestoque = null;
        }
        if (!($pedido['NRINSJURTRAN'])){
            $pedido['NRINSJURTRAN'] = $paramsExp[0]['NRINSJURTRAN'];
            $tranportadora = $this->ExpedicaoService->getTransportadora($pedido['NRINSJURTRAN']);
        }
        if (!($pedido['IDTPIJURTRAN'])){
            if ($paramsExp[0]['IDTPIJURTRAN']){
                $pedido['IDTPIJURTRAN'] = $paramsExp[0]['IDTPIJURTRAN'];
            } else if (isset($tranportadora[0])){
                $pedido['IDTPIJURTRAN'] = $tranportadora[0]['IDTPIJURTRAN'];
            }
        }
        $sgufveic = null;
        $nrplacaveic = null;
        if ($pedido['CDVIATURA'] && !(isset($pedido['NRPLACAVEIC'])) && !(isset($pedido['SGUFVEIC']))){
            if($viatura && $viatura[0] && $viatura[0]['NRPLACA']){
                $nrplacaveic = $viatura[0]['NRPLACA'];
                $sgufveic = $viatura[0]['SGUFVEICULO'];
            }
        }

        $nrlanctonf = $this->getNewCode('LANCNF/FILIAL', $cdfilial);
        $nrlancestq = $this->getNewCode('LANCTOEST', $cdfilial);

        $corpoNF = array(
            'P_CDFILIAL'    => $cdfilial,
            'P_CDFILIEMIT'  => $cdfilial,
            'P_CDEMPRESA'   => $pedido['CDEMPRESA'],
            'P_CDTIPOOPER'  => $pedido['CDTIPOOPER'], //(tem que ver parametro para tb pegar do usuario(filtro)(CDTIPOOPER -> PEDIDO),
            'P_CDSERIE'     => $pedido['CDSERIE'],
            'P_CDCLIENTE'   => $p_cdcliente,
            'P_DTENTRSAID'  => $dtProcessa,
            'P_IDSISTEMA'   => 'EXP',
            'P_ATUAESTOQUE' => $p_atuaestoque,
            'P_NRLANCTONF'  => $nrlanctonf,
            'P_NRAPURACFAT' => null,
            'P_IDTPIJURTRAN'=> $pedido['IDTPIJURTRAN'], // Setar esse campo no frontend, se chegar nulo, olho o parametro
            'P_NRINSJURTRAN'=> $pedido['NRINSJURTRAN'], // Setar esse campo no frontend, se chegar nulo, olho o parametro
            'P_NRPLACAVEIC' => $nrplacaveic,
            'P_SGUFVEIC'    => $sgufveic,
            'P_IDPGTOFRETE' => $pedido['IDPGFRETTRAN'], // Setar esse campo no frontend
            'P_DSQTDEVOLU'  => null,
            'P_DSESPECVOLU' => $pedido['DSESPEVOLPED'],
            'P_DSMARCAVOLU' => $pedido['DSMARCVOLPED'],
            'P_DSNUMEVOLU'  => $pedido['DSNUMEVOLPED'],
            'P_VRPBRUTOVOLU'=> null,
            'P_VRPLIQUVOLU' => null,
            'P_NRLANCORIG'  => '',
            'P_NRLANCESTQ'  => $nrlancestq,
            'P_DTLANCAMENNF'=> $sysdate
        );
        return $corpoNF;
    }

    public function getNewCode($cdcontador, $cdfilial) {
        $newCode = $this->newCode->getCodeByTableName($cdcontador.$cdfilial, $this->environment->getNrOrgTrab());
        return $newCode;
    }

    public function geraItemNf($pedido, $item, $dadosFilial, $cdfilial, $nrpedido, $sysdate, $nrlanctonf, $lotesItemFront){

        if (!(count($lotesItemFront))) {
            
            $paramsFilial   = $this->ExpedicaoService->getParamsFilial($cdfilial);
            $tipooper       = $this->ExpedicaoService->getTipooperac($pedido['CDTIPOOPER']);
            
            if($paramsFilial && $paramsFilial[0]['IDCTRLESTQ'] == 'S' &&
               $tipooper    && $tipooper[0]['IDATUESTTPOP'] == 'S' &&
               $dadosFilial && $dadosFilial[0]['IDUTILALMOX'] == 'S'){
                        
                if ($dadosFilial[0]['IDUTILALMOX'] == 'S'){
                    $lotesItem = $this->reservaPedidoService->buscaEstoqueAlmox($cdfilial, $item['NRPEDIDO'], $item['NRITEMPED'], $item['CDPRODESTO'], $item['CDALMOXARIFE'], $item['CDLOCALESTOQ']);
                } else {
                    $lotesItem = $this->reservaPedidoService->buscaEstoque($cdfilial, $item['NRPEDIDO'], $item['NRITEMPED'], $item['CDPRODESTO'], $item['CDALMOXARIFE'], $item['CDLOCALESTOQ']);
                }
                if (count($lotesItem) === 0){
                    //throw new \Exception('Item '.$item['NMPRODUTO'].' do pedido '.$pedido['NRPEDIDO'].' - '.$pedido['NMRAZSOCCLIE'].' não possui lote.');
                    throw new \Exception("erro_lote/".$item['NMPRODUTO']."/".$pedido['NRPEDIDO']."/".$pedido['NMRAZSOCCLIE']);
                }
            }else{
                $lotesItem[0]['NRLOTEESTQ'] = '';
                $lotesItem[0]['NRSUBLOTE'] = '';
            } 
        } 
        
        if (!(empty($item['CDEMISSAOITPED']))) {
            $stCdProdEmiss = $item['CDEMISSAOITPED'];
        } else {
            $stCdProdEmiss = $item['CDARVPROD'];
        }
        
        if (!(empty($item['DSEMISSAOITPED']))) {
            $stDsProdEmiss = $item['DSEMISSAOITPED'];
        } else {
            $stDsProdEmiss = $item['NMPRODUTO'];
        }
        
        /*  if (count($lotesItemFront) > 1){
            //Chama a função de dividir a expedição do item de forma 'manual', passando os lotes e as quantidades cadastradas, manualmente, no Frontend para os itens da nota.
            $produtoNF = $this->divideItemLoteManual($item, $lotesItemFront, $cdfilial, $nrlanctonf, $stCdProdEmiss, $stDsProdEmiss);
            
        } else if ($lotesItem[0]['QTDISPONIVEL'] < $item['QTITEMPED']){
            //Chama a função de dividir o a expedição do item de forma 'automatica', passando os lotes validos do produto e dividindo um item da nota quantas vezes for necessario.
            $produtoNF = $this->divideItemLoteAuto($item, $lotesItem, $cdfilial, $nrlanctonf, $stCdProdEmiss, $stDsProdEmiss);
            
        } else {*/
            $produtoNF = array(
                array(
                    'P_CDFILIAL'        => $cdfilial,
                    'P_NRLANCTONF'      => $nrlanctonf,
                    'P_CDPRODUTO'       => $item['CDPRODESTO'],
                    'P_CDPRODEMIS'      => $stCdProdEmiss,
                    'P_DSPRODEMIS'      => $stDsProdEmiss,
                    'P_CDFILIALITG'     => $cdfilial,
                    'P_NRINTEGRACAO'    => $item['NRPEDIDO'],
                    'P_NRITEMPED'       => $item['NRITEMPED'],
                    'P_CDALMOXARIFE'    => $item['CDALMOXARIFE'],       // serviço de estoque
                    'P_CDLOCALESTOQ'    => $item['CDLOCALESTOQ'],       // serviço de estoque
                    'P_NRLOTEESTQ'      => $lotesItem[0]['NRLOTEESTQ'], // serviço de estoque, Quando o 1 lote trazido nao tiver quantidade suficiente, chamar procedure 1 por lote
                    'P_NRSUBLOTE'       => $lotesItem[0]['NRSUBLOTE'],  // Vem com o lote, se nao "controlar" vem ' '
                    'P_QTITEMPROD'      => $item['QTITEMPED'],          // Pega o maximo que o lote permitir, ou o que o pedido pede
                    'P_VRUNITITEM'      => $item['VRITEMPEDCONVERT'],   // ...
                    'P_VRDESCITEM'      => $item['VRDESITPEDCONVERT'],  //    `-> Valores convertidos para a moeda da tabela de preço
                    'P_IDCONFORITEM'    => 'S',
                    'P_QTVOLUITEM'      => '0',
                    'P_IDORIGEMPROD'    => 'N',
                    'P_IDATUESTPROD'    => 'S',
                    'P_VRICMSSTITORIG'  => '0'
                )
            );
        // }
        return $produtoNF;
    }

    public function divideItemLoteManual($item, $lotesItem, $cdfilial, $nrlanctonf, $stCdProdEmiss, $stDsProdEmiss){

        for ($i = 0; $i < count($lotesItem); $i++) {
            $qtitemLote = $lotesItem[$i]['QTDISPONIVEL'];
            $produto = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRLANCTONF'      => $nrlanctonf,
                'P_CDPRODUTO'       => $item['CDPRODESTO'],
                'P_CDPRODEMIS'      => $stCdProdEmiss,
                'P_DSPRODEMIS'      => $stDsProdEmiss,
                'P_CDFILIALITG'     => $cdfilial,
                'P_NRINTEGRACAO'    => $item['NRPEDIDO'],
                'P_NRITEMPED'       => $item['NRITEMPED'],
                'P_CDALMOXARIFE'    => $item['CDALMOXARIFE'], // serviço de estoque
                'P_CDLOCALESTOQ'    => $item['CDLOCALESTOQ'], // serviço de estoque
                'P_NRLOTEESTQ'      => $lotesItem[$i]['NRLOTEESTQ'],   // Utiliza o cadastrado no frontend
                'P_NRSUBLOTE'       => $lotesItem[$i]['NRSUBLOTE'],    // Utiliza o cadastrado no frontend
                'P_QTITEMPROD'      => $lotesItem[$i]['QTRESPRODPED'], // Utiliza o cadastrado no frontend
                'P_VRUNITITEM'      => $item['VRITEMPEDCONVERT'],
                'P_VRDESCITEM'      => $item['VRDESITPEDCONVERT'],
                'P_IDCONFORITEM'    => 'S',
                'P_QTVOLUITEM'      => '0',
                'P_IDORIGEMPROD'    => 'N',
                'P_IDATUESTPROD'    => 'S',
                'P_VRICMSSTITORIG'  => '0'
            );
            $produtoNF[$i] = $produto;
        }
        return $produtoNF;
    }

    public function divideItemLoteAuto($item, $lotesItem, $cdfilial, $nrlanctonf, $stCdProdEmiss, $stDsProdEmiss){

        $qtitem = $item['QTITEMPED'];
        $qtRest = $lotesItem[0]['QTDISPONIVEL'];
        $produtoNF = array();

        for ($i = 0; $i < count($lotesItem); $i++) {
            $qtitemLote = $lotesItem[$i]['QTDISPONIVEL'];
            $produto = array(
                'P_CDFILIAL'        => $cdfilial,
                'P_NRLANCTONF'      => $nrlanctonf,
                'P_CDPRODUTO'       => $item['CDPRODESTO'],
                'P_CDPRODEMIS'      => $stCdProdEmiss,
                'P_DSPRODEMIS'      => $stDsProdEmiss,
                'P_CDFILIALITG'     => $cdfilial,
                'P_NRINTEGRACAO'    => $item['NRPEDIDO'],
                'P_NRITEMPED'       => $item['NRITEMPED'],
                'P_CDALMOXARIFE'    => $item['CDALMOXARIFE'], // serviço de estoque
                'P_CDLOCALESTOQ'    => $item['CDLOCALESTOQ'], // serviço de estoque
                'P_NRLOTEESTQ'      => $lotesItem[$i]['NRLOTEESTQ'], // serviço de estoque, Quando o 1 lote trazido nao tiver quantidade suficiente, chamar procedure 1 por lote
                'P_NRSUBLOTE'       => $lotesItem[$i]['NRSUBLOTE'],  // Vem com o lote, se nao "controlar" vem ' '
                'P_QTITEMPROD'      => $qtRest, // Pega o maximo que o lote permitir, ou o que o pedido pede
                'P_VRUNITITEM'      => $item['VRITEMPEDCONVERT'],
                'P_VRDESCITEM'      => $item['VRDESITPEDCONVERT'],
                'P_IDCONFORITEM'    => 'S',
                'P_QTVOLUITEM'      => '0',
                'P_IDORIGEMPROD'    => 'N',
                'P_IDATUESTPROD'    => 'S',
                'P_VRICMSSTITORIG'  => '0'
            );
            $produtoNF[$i] = $produto;
            $qtRest = $qtitem - $qtitemLote;
            if($qtRest === 0) {
                $i = count($lotesItem);
            }
        }

        return $produtoNF;
    }

    public function callProcedureGERA_NOTAFISCAL_CORPO($corpoNF){
        try {
            return $this->ExpedicaoService->callProcedureGERA_NOTAFISCAL_CORPO($corpoNF);
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function callProcedureGERA_NOTAFISCAL_PRODUTO($produtoNF){
        try {
            return $this->ExpedicaoService->callProcedureGERA_NOTAFISCAL_PRODUTO($produtoNF);
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function callProcedureGERA_NOTAFISCAL_ENCARGO($encargoNF){
        try {
            return $this->ExpedicaoService->callProcedureGERA_NOTAFISCAL_ENCARGO($encargoNF);
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function callProcedureGERA_NOTAFISCAL_DADOSADIC($dadosadicNF){
        try {
            return $this->ExpedicaoService->callProcedureGERA_NOTAFISCAL_DADOSADIC($dadosadicNF);
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }

    public function callProcedureGERA_NOTAFISCAL_FECHAMENTO($fechamentoNF){
        try {
            return $this->ExpedicaoService->callProcedureGERA_NOTAFISCAL_FECHAMENTO($fechamentoNF);
        }catch(\Exception $e){
            return $e->getMessage();
        }
    }
    
    public function resultValue($field){
        if (($field == '') || ($field == null)){
            return ' ';
        }else 
            return $field;
    }
    
    public function salvaColeta($request, $response){
        
        /* Inicia Connection */
        $this->entityManager->getConnection()->beginTransaction();
        
        try {
            $row = $request->getRow();
            $cdfilial           = $row[0]['CDFILIAL'];
            $nrlanctonf         = $row[0]['NRLANCTONF'];
            $cdproduto          = $row[0]['CDPRODUTO'];
            $nrloteestq         = $row[0]['NRLOTEESTQ'];
            $qtresprodped       = $row[0]['QTRESPRODPED'];
            $nrseqcontainer     = $row[0]['NRSEQCONTAINER'];
            $qtcontainer        = $row[0]['QTCONTAINER'];
            $idstatuscol        = $row[0]['IDSTATUSCOL'];
            $nrsublote          = null;
            
            
            //  Verifica se o produto compõe a Pre nota 
            $produtoPrenota = $this->ExpedicaoService->verificaProdutoCompoeNota($cdfilial, $nrlanctonf, $cdproduto);
                
            if(count($produtoPrenota)){
                
                //  Verificar se a filial controla almoxarifado/localização e se tem almoxarifado/localização de expedição cadastrado 
                $idutilalmox    = $produtoPrenota[0]['IDUTILALMOX'];
                $cdalmoxprcli   = $produtoPrenota[0]['CDALMOXPRCLI'];
                $idutillcestq   = $produtoPrenota[0]['IDUTILLCESTQ'];
                $cdlocalprcli   = $produtoPrenota[0]['CDLOCALPRCLI'];
                $nrseqitprod    = $produtoPrenota[0]['NRSEQITPROD'];
                
                if(($idutilalmox === 'S') && (is_null($cdalmoxprcli))){
                    throw new \Exception("Almoxarifado de Expedição deve ser parametrizado");
                }else if(($idutillcestq == 'S') && (is_null($cdlocalprcli))){
                    throw new \Exception("Localização de Expedição deve ser parametrizado");
                }else{
                    
                    //  Verificar se existe o lote informado 
                    $lotePrenota = $this->ExpedicaoService->verificaLoteNota($cdfilial, $cdproduto, $nrloteestq, $nrsublote);
                    if(count($lotePrenota)){
                    $newLote = $lotePrenota[0]['NRLOTEESTQ'];
                    
                    $nrloteestq = $newLote; 
                        
                        // Busca quantidade total já coletada
                        $qtcoletada = $this->ExpedicaoService->getQtColetadaVol($cdfilial, $nrlanctonf, $nrseqcontainer);
                        $qtcoletada = $qtcoletada[0]['TOTQTCOLETADA'];
                        $qtTotcoletada = $qtcoletada+$qtresprodped;

                        if($qtTotcoletada <= $qtcontainer){
                            /*if($qtTotcoletada > 0){
                                $this->ExpedicaoService->updateStVolumeConcluido($cdfilial, $nrlanctonf, $nrseqcontainer, 'AN');
                            }*/
                            
                            // Atualiza quantidade já coletada no volume
                            $this->ExpedicaoService->updateQtColetadaVol($qtresprodped, $cdfilial, $nrlanctonf, $nrseqcontainer);
                            
                            $nrseqcoleta = $this->ExpedicaoService->getMaxNrseqcoleta($cdfilial, 
                                                                                        $nrlanctonf, 
                                                                                        $nrseqcontainer);
                            $nrseqcoleta = $nrseqcoleta[0]['NRSEQCOLETA'];
                    
                            // Insere Coleta
                            $this->ExpedicaoService->insertColeta($cdfilial, 
                                                                $nrlanctonf, 
                                                                $nrseqcontainer, 
                                                                $nrseqcoleta,  // sequencial
                                                                $cdproduto, 
                                                                $this->resultValue($nrloteestq), 
                                                                $this->resultValue($nrsublote), 
                                                                $qtresprodped);
                    
                            // Busca ItemProduto
                            $itensProduto = $this->ExpedicaoService->getItemproduto($cdfilial, $nrlanctonf, $cdproduto);
                            if(count($itensProduto)){
                            // INICIO SE TEM ITEMPRODUTO
                                $rownum = 1;
                                foreach ($itensProduto as $itemP) {
                                // INICIO FOREACH DE ITEMPRODUTO
                                    $qtFaltaSepIt = $itemP['QTEXPEDIR'] - $itemP['QTSEPARADA'];
                                    if($qtresprodped > $qtFaltaSepIt){
                                    // INICIO SE QUANTIDADE A EXPEDIR FOR MAIOR QUE A QUE FALTA
                    
                                        if($rownum == count($itensProduto)){ // (adqItemProd.RecNo = adqItemProd.RecordCount)
                                            $qtFaltaSepIt = 0;
                                            $qtSeparada = $itemP['QTSEPARADA'] + $qtresprodped;
                                            
                                            // Atualiza situação do produto na ITEMPRODUTO
                                            $this->ExpedicaoService->updateStItemproduto($cdfilial, 
                                                                                        $nrlanctonf, 
                                                                                        $itemP['NRSEQITPROD'],
                                                                                        $qtSeparada,
                                                                                        'LQ');
                                            // Busca RESERITEMPROD
                                            $reservaItemprod = $this->ExpedicaoService->getReservaItemprod($cdfilial, 
                                                                                                        $nrlanctonf, 
                                                                                                        $itemP['NRSEQITPROD'], 
                                                                                                        $itemP['CDPRODUTO'], 
                                                                                                        $this->resultValue($itemP['CDALMOXARIFE']), 
                                                                                                        $this->resultValue($itemP['CDLOCALESTOQ']), 
                                                                                                        $this->resultValue($nrloteestq), 
                                                                                                        $this->resultValue($nrsublote));
                                            
                                            if(count($reservaItemprod)){
                                                // Updata reserva caso já exista
                                                $this->ExpedicaoService->updateReservaItemprodSoma($qtresprodped,
                                                                                                    $cdfilial, 
                                                                                                    $nrlanctonf, 
                                                                                                    $itemP['NRSEQITPROD'],
                                                                                                    $reservaItemprod[0]['NRRESPRODPED']);
                                            }else{
                                                // Gera sequencial da coleta 
                                                $nrresprodped = $this->ExpedicaoService->getMaxNrresprodped($cdfilial, 
                                                                                                            $nrlanctonf, 
                                                                                                            $itemP['NRSEQITPROD']);
                                                $nrresprodped = $nrresprodped[0]['NRRESPRODPED'];
                                                
                                                // Insere reserva caso não exista
                                                $this->ExpedicaoService->insereReservaItemprod($cdfilial,
                                                                                                $nrlanctonf,
                                                                                                $itemP['NRSEQITPROD'],
                                                                                                $nrresprodped,
                                                                                                $itemP['CDPRODUTO'],
                                                                                                $this->resultValue($itemP['CDALMOXARIFE']),
                                                                                                $this->resultValue($itemP['CDLOCALESTOQ']),
                                                                                                $this->resultValue($nrloteestq),
                                                                                                $this->resultValue($nrsublote),
                                                                                                $qtresprodped,
                                                                                                '');
                                            }
                                            
                                            $qtresprodped = 0;
                                        }else{
                                            $qtresprodped = $qtresprodped - $qtFaltaSepIt;
                                            $qtSeparada = $itemP['QTSEPARADA'] + $qtFaltaSepIt;
                                            $qtFaltaSepIt = 0;
                                            
                                            // Atualiza situação do produto na ITEMPRODUTO
                                            $this->ExpedicaoService->updateStItemproduto($cdfilial, 
                                                                                        $nrlanctonf, 
                                                                                        $itemP['NRSEQITPROD'],
                                                                                        $qtSeparada,
                                                                                        'LQ');

                                            // Busca RESERITEMPROD
                                            $reservaItemprod = $this->ExpedicaoService->getReservaItemprod($cdfilial, 
                                                                                                        $nrlanctonf, 
                                                                                                        $itemP['NRSEQITPROD'], 
                                                                                                        $itemP['CDPRODUTO'], 
                                                                                                        $this->resultValue($itemP['CDALMOXARIFE']), 
                                                                                                        $this->resultValue($itemP['CDLOCALESTOQ']), 
                                                                                                        $this->resultValue($nrloteestq), 
                                                                                                        $this->resultValue($nrsublote));
                                                                                                        
                                            if(count($reservaItemprod)){
                                                // Updata reserva caso já exista
                                                $this->ExpedicaoService->updateReservaItemprod($qtSeparada,
                                                                                                $cdfilial, 
                                                                                                $nrlanctonf, 
                                                                                                $itemP['NRSEQITPROD'],
                                                                                                $reservaItemprod[0]['NRRESPRODPED']);
                                                                                                
                                            }else{
                                                // Gera sequencial da coleta 
                                                $nrresprodped = $this->ExpedicaoService->getMaxNrresprodped($cdfilial, 
                                                                                                            $nrlanctonf, 
                                                                                                            $itemP['NRSEQITPROD']);
                                                $nrresprodped = $nrresprodped[0]['NRRESPRODPED'];
                                                
                                                // Insere reserva caso não exista
                                                $this->ExpedicaoService->insereReservaItemprod($cdfilial, 
                                                                                                $nrlanctonf, 
                                                                                                $itemP['NRSEQITPROD'],
                                                                                                $nrresprodped,
                                                                                                $itemP['CDPRODUTO'],
                                                                                                $this->resultValue($itemP['CDALMOXARIFE']),
                                                                                                $this->resultValue($itemP['CDLOCALESTOQ']),
                                                                                                $this->resultValue($nrloteestq), 
                                                                                                $this->resultValue($nrsublote),
                                                                                                $qtSeparada,
                                                                                                '');
                                                                                                
                                            }
                                        }
                                        
                                    // FIM SE QUANTIDADE A EXPEDIR FOR MAIOR QUE A QUE FALTA
                                    }else{
                                    // INICIO SE QUANTIDADE A EXPEDIR FOR MENOR QUE A QUANTIDADE QUE FALTA
                                    
                                        $qtSeparada = $itemP['QTSEPARADA'] + $qtresprodped;//12
                                        $qtFaltaSepIt = $itemP['QTEXPEDIR'] - $qtSeparada; //38
                                        $boAcabou = true;
                                    
                                        // boAcabou := True;
                                        
                                        if($qtFaltaSepIt > 0){
                                            $idstatusreser = 'PA';
                                        }else{
                                            $idstatusreser = 'LQ';
                                        }
                    
                                        $this->ExpedicaoService->updateStItemproduto($cdfilial, 
                                                                                        $nrlanctonf, 
                                                                                        $itemP['NRSEQITPROD'],
                                                                                        $qtSeparada,
                                                                                        $idstatusreser);
                                        // Busca RESERITEMPROD
                                        $reservaItemprod = $this->ExpedicaoService->getReservaItemprod($cdfilial, 
                                                                                                        $nrlanctonf, 
                                                                                                        $itemP['NRSEQITPROD'], 
                                                                                                        $itemP['CDPRODUTO'], 
                                                                                                        $this->resultValue($itemP['CDALMOXARIFE']), 
                                                                                                        $this->resultValue($itemP['CDLOCALESTOQ']), 
                                                                                                        $this->resultValue($nrloteestq), 
                                                                                                        $this->resultValue($nrsublote));
                    
                                        if(count($reservaItemprod)){
                                            
                                            // Updata reserva caso já exista
                                            $this->ExpedicaoService->updateReservaItemprod($qtSeparada,
                                                                                            $cdfilial, 
                                                                                            $nrlanctonf, 
                                                                                            $itemP['NRSEQITPROD'],
                                                                                            $reservaItemprod[0]['NRRESPRODPED']);
                                        }else{
                                            // Gera sequencial da coleta 
                                            $nrresprodped = $this->ExpedicaoService->getMaxNrresprodped($cdfilial, 
                                                                                                        $nrlanctonf, 
                                                                                                        $itemP['NRSEQITPROD']);
                                            $nrresprodped = $nrresprodped[0]['NRRESPRODPED'];
                                            
                    
                                            // Insere reserva caso não exista
                                            $this->ExpedicaoService->insereReservaItemprod($cdfilial, 
                                                                                            $nrlanctonf, 
                                                                                            $itemP['NRSEQITPROD'],
                                                                                            $nrresprodped,
                                                                                            $itemP['CDPRODUTO'],
                                                                                            $this->resultValue($itemP['CDALMOXARIFE']),
                                                                                            $this->resultValue($itemP['CDLOCALESTOQ']),
                                                                                            $this->resultValue($nrloteestq), 
                                                                                            $this->resultValue($nrsublote),
                                                                                            $qtresprodped,
                                                                                            '');
                    
                                        }
                                        
                                        if($boAcabou){
                                            break;
                                        }
                                        
                                    // FIM SE QUANTIDADE A EXPEDIR FOR MENOR QUE A QUANTIDADE QUE FALTA
                                    }
                                    
                                    $rownum++;
                                    
                                // FIM FOREACH DE ITEMPRODUTO
                                }
                                
                            // FIM SE TEM PRODUTO
                            }else{
                                
                                $maxItemprod = $this->ExpedicaoService->getMaxItemprod($cdfilial, $nrlanctonf, $cdproduto);
                                
                                if($maxItemprod){
                                    
                                    $this->ExpedicaoService->updateStItemproduto($cdfilial, 
                                                                                $nrlanctonf, 
                                                                                $maxItemprod[0]['NRSEQITPROD'],
                                                                                $maxItemprod[0]['QTSEPARADA'] + $qtresprodped,
                                                                                'LQ');
                                                                                
                                    // Busca RESERITEMPROD
                                    $reservaItemprod = $this->ExpedicaoService->getReservaItemprod($cdfilial, 
                                                                                                    $nrlanctonf, 
                                                                                                    $maxItemprod[0]['NRSEQITPROD'], 
                                                                                                    $maxItemprod[0]['CDPRODUTO'], 
                                                                                                    $this->resultValue($maxItemprod[0]['CDALMOXARIFE']), 
                                                                                                    $this->resultValue($maxItemprod[0]['CDLOCALESTOQ']), 
                                                                                                    $this->resultValue($nrloteestq), 
                                                                                                    $this->resultValue($nrsublote));
                        
                                    if(count($reservaItemprod)){
                                        // Updata reserva caso já exista
                                        $this->ExpedicaoService->updateReservaItemprodSoma($qtresprodped,
                                                                                            $cdfilial, 
                                                                                            $nrlanctonf, 
                                                                                            $maxItemprod[0]['NRSEQITPROD'],
                                                                                            $reservaItemprod[0]['NRRESPRODPED']);
                                    }else{
                                        
                                        // Gera sequencial da coleta 
                                        $nrresprodped = $this->ExpedicaoService->getMaxNrresprodped($cdfilial, 
                                                                                                    $nrlanctonf, 
                                                                                                    $maxItemprod[0]['NRSEQITPROD']);
                                        $nrresprodped = $nrresprodped[0]['NRRESPRODPED'];
                                        
                                        // Insere reserva caso não exista
                                        $this->ExpedicaoService->insereReservaItemprod($cdfilial, 
                                                                                        $nrlanctonf, 
                                                                                        $maxItemprod[0]['NRSEQITPROD'],
                                                                                        $nrresprodped,
                                                                                        $maxItemprod[0]['CDPRODUTO'],
                                                                                        $this->resultValue($maxItemprod[0]['CDALMOXARIFE']),
                                                                                        $this->resultValue($maxItemprod[0]['CDLOCALESTOQ']),
                                                                                        $this->resultValue($nrloteestq), 
                                                                                        $this->resultValue($nrsublote),
                                                                                        $qtresprodped,
                                                                                        '');
                                    }
                                    
                                    $qtresprodped = 0;
                                
                                }
                                
                            }
                            
                        }else{
                            throw new \Exception("A capacidade do volume foi excedida!
                                                <br><br>Capacidade máxima: ".$qtcontainer.
                                                "<br>Quantidade total coletada: ".$qtTotcoletada);
                        }
                        
                    }else{
                        throw new \Exception("Atenção! Lote não encontrado");
                    }
                    
                }
                
                
            }else{
                throw new \Exception("Leitura inválida! Produto: ".$row[0]['CDPRODUTO']." não faz parte da pré-nota.");
            }

            // Valida se excedeu a coleta prevista.
            $coletaExced = $this->validaColetaExced($cdfilial, $nrlanctonf, $nrseqcontainer);
            
            // Valida situação do volume de acordo com coleta realizada.
            $status = $this->validaSitVolume($cdfilial, $nrlanctonf, $nrseqcontainer);
            
            // Valida se existe coleta para a pre NF selecionada, se existe updata a NF pra "Em andamento."
            $this->validaSitPrenf($cdfilial, $nrlanctonf);
            
            // Retorna dataSet pro frontend
            $data = array('STATUS' => $status, 'COLETAEXCED' => $coletaExced);
            $dataSet = new DataSet ("data", $data);
            $response->addDataSet($dataSet);

        /* Commit */
        $this->entityManager->getConnection()->commit();

        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
            
            /* rollback */
            $this->entityManager->rollback();
        }
    }
    
    public function validaColetaExced($cdfilial, $nrlanctonf, $nrseqcontainer){
        $coletas = $this->ExpedicaoService->getColetaExcedida($cdfilial, $nrlanctonf, $nrseqcontainer);
            
        $coletaExced = '';
        if(count($coletas)){
            foreach ($coletas as $coleta) {
                   
                // Se tiver alguma coleta com quantidade excedida alem da % permitida, retorna para usuário.
                if($coleta['IDCOLETA'] === 'E'){
                    //throw new \Exception("Coleta exceceu o % máximo para expedição adicional.");
                    $coletaExced = "S";
                }
                
            }
        }
        return $coletaExced;
    } 
    
    public function validaSitVolume($cdfilial, $nrlanctonf, $nrseqcontainer){
        $qtcoletada = $this->ExpedicaoService->getQtColetadaVol($cdfilial, $nrlanctonf, $nrseqcontainer);
        
        if(count($qtcoletada[0]['TOTQTCOLETADA'])){
            $this->ExpedicaoService->updateStVolumeConcluido($cdfilial, $nrlanctonf, $nrseqcontainer, 'AN');
            $status = 'AN';
        }else{
            $this->ExpedicaoService->updateStVolumeConcluido($cdfilial, $nrlanctonf, $nrseqcontainer, 'PE');
            $status = 'PE';
        }
        return $status;
    }
    
    public function validaSitPrenf($cdfilial, $nrlanctonf){
        $qtcoletada = $this->ExpedicaoService->getQtColetadaNF($cdfilial, $nrlanctonf);
        
        if(count($qtcoletada[0]['TOTQTCOLETADA'])){
            $this->ExpedicaoService->updateStNfColetada($cdfilial, $nrlanctonf, 'A');
        }else{
            $this->ExpedicaoService->updateStNfColetada($cdfilial, $nrlanctonf, 'P');
        }
    }
    
    public function excluiColeta($request, $response){
        
        /* Inicia Connection */
        $this->entityManager->getConnection()->beginTransaction();
        
        try {
            $rows = $request->getRow();
            $cdfilial = $rows[0]['CDFILIAL'];
            $nrlanctonf = $rows[0]['NRLANCTONF'];
            $cdproduto = $rows[0]['CDPRODUTO'];
            $nrseqcontainer = $rows[0]['NRSEQCONTAINER'];
            
            $idstatuscol    = $this->ExpedicaoService->getStContainer($cdfilial, $nrlanctonf, $nrseqcontainer);
            $idstatuscol    = $idstatuscol[0]['IDSTATUSCOL'];
            
            foreach ($rows as $row) {
                // DELETA AS COLETAS MARCADAS
                $this->ExpedicaoService->deleteColetaVol($cdfilial,
                                                        $nrlanctonf,
                                                        $nrseqcontainer,
                                                        $row['NRSEQCOLETA']);
                
                // A CADA COLETA DELETADA, UPDATA QUANTIDADE COLETADA ATUAL NO VOLUME      
                $this->ExpedicaoService->updateQtColetadaVolDel($row['QTCOLETADA'],
                                                                $cdfilial,
                                                                $nrlanctonf,
                                                                $nrseqcontainer);
            }
            
            // DELETA TODAS AS RESERVAS PARA REFAZER A ROTINA DE RESERVA
            $this->ExpedicaoService->deleteReservaItemprod($cdfilial, $nrlanctonf, $cdproduto);
            
            // UPDATA ITEMPRODUTO SETANDO QUANTIDADE COLETADA PARA 0, E STATUS PARA PENDENTE, PARA ENTÃO REFAZER A ROTINA DE RESERVA
            $this->ExpedicaoService->updateStItemprodutoDel($cdfilial, $nrlanctonf, $cdproduto, $qtseparada = 0, $idstatusreser = 'PE');
            
            // INICIO ROTINA DE RESERVA SEMELHANTE NO SALVA COLETA            
            $coletasVol = $this->ExpedicaoService->getColetaVol($cdfilial, $nrlanctonf, $cdproduto);
            if(count($coletasVol)){
                
                foreach ($coletasVol as $coleta) {
                    
                    $qtresprodped = $coleta['QTCOLETADA'];
                    
                    $itensProduto = $this->ExpedicaoService->getItemproduto($cdfilial, $nrlanctonf, $cdproduto);
                    if(count($itensProduto)){
                    // INICIO SE TEM ITEMPRODUTO
                        $rownum = 1;
                        foreach ($itensProduto as $itemP) {
                        // INICIO FOREACH DE ITEMPRODUTO
                            $qtFaltaSepIt = $itemP['QTEXPEDIR'] - $itemP['QTSEPARADA'];
                            if($qtresprodped > $qtFaltaSepIt){
                            // INICIO SE QUANTIDADE A EXPEDIR FOR MAIOR QUE A QUE FALTA
            
                                if($rownum == count($itensProduto)){ // (adqItemProd.RecNo = adqItemProd.RecordCount)
                                    $qtSeparada = $itemP['QTSEPARADA'] + $qtresprodped;
                                    $qtFaltaSepIt = 0;
                                    
                                    // Atualiza situação do produto na ITEMPRODUTO
                                    $this->ExpedicaoService->updateStItemproduto($cdfilial, 
                                                                                $nrlanctonf, 
                                                                                $itemP['NRSEQITPROD'],
                                                                                $qtSeparada,
                                                                                'LQ');
                                    // Busca RESERITEMPROD
                                    $reservaItemprod = $this->ExpedicaoService->getReservaItemprod($cdfilial, 
                                                                                                $nrlanctonf, 
                                                                                                $itemP['NRSEQITPROD'], 
                                                                                                $itemP['CDPRODUTO'], 
                                                                                                $this->resultValue($itemP['CDALMOXARIFE']), 
                                                                                                $this->resultValue($itemP['CDLOCALESTOQ']), 
                                                                                                $this->resultValue($coleta['NRLOTEESTQ']), 
                                                                                                $this->resultValue($coleta['NRSUBLOTE']));
                                    
                                    if(count($reservaItemprod)){
                                        // Updata reserva caso já exista
                                        $this->ExpedicaoService->updateReservaItemprodSoma($qtresprodped,
                                                                                            $cdfilial, 
                                                                                            $nrlanctonf, 
                                                                                            $itemP['NRSEQITPROD'],
                                                                                            $reservaItemprod[0]['NRRESPRODPED']);
                                    }else{
                                        // Gera sequencial da coleta 
                                        $nrresprodped = $this->ExpedicaoService->getMaxNrresprodped($cdfilial, 
                                                                                                    $nrlanctonf, 
                                                                                                    $itemP['NRSEQITPROD']);
                                        $nrresprodped = $nrresprodped[0]['NRRESPRODPED'];
                                        
                                        // Insere reserva caso não exista
                                        $this->ExpedicaoService->insereReservaItemprod($cdfilial, 
                                                                                        $nrlanctonf, 
                                                                                        $itemP['NRSEQITPROD'],
                                                                                        $nrresprodped,
                                                                                        $itemP['CDPRODUTO'],
                                                                                        $this->resultValue($itemP['CDALMOXARIFE']),
                                                                                        $this->resultValue($itemP['CDLOCALESTOQ']),
                                                                                        $this->resultValue($coleta['NRLOTEESTQ']), 
                                                                                        $this->resultValue($coleta['NRSUBLOTE']),
                                                                                        $qtresprodped,
                                                                                        '');
                                    }
                                    
                                    $qtresprodped = 0;
                                }else{
                                    $qtresprodped = $qtresprodped - $qtFaltaSepIt;
                                    $qtSeparada = $itemP['QTSEPARADA'] + $qtFaltaSepIt;
                                    $qtFaltaSepIt = 0;
                                    
                                    // Atualiza situação do produto na ITEMPRODUTO
                                    $this->ExpedicaoService->updateStItemproduto($cdfilial, 
                                                                                $nrlanctonf, 
                                                                                $itemP['NRSEQITPROD'],
                                                                                $qtSeparada,
                                                                                'LQ');
                                
                                    // Busca RESERITEMPROD
                                    $reservaItemprod = $this->ExpedicaoService->getReservaItemprod($cdfilial, 
                                                                                                $nrlanctonf, 
                                                                                                $itemP['NRSEQITPROD'], 
                                                                                                $itemP['CDPRODUTO'], 
                                                                                                $this->resultValue($itemP['CDALMOXARIFE']), 
                                                                                                $this->resultValue($itemP['CDLOCALESTOQ']), 
                                                                                                $this->resultValue($coleta['NRLOTEESTQ']), 
                                                                                                $this->resultValue($coleta['NRSUBLOTE']));
                                                                                                
                                    if(count($reservaItemprod)){
                                        // Updata reserva caso já exista
                                        $this->ExpedicaoService->updateReservaItemprod($qtSeparada,
                                                                                        $cdfilial, 
                                                                                        $nrlanctonf, 
                                                                                        $itemP['NRSEQITPROD'],
                                                                                        $reservaItemprod[0]['NRRESPRODPED']);
                                                                                        
                                    }else{
                                        // Gera sequencial da coleta 
                                        $nrresprodped = $this->ExpedicaoService->getMaxNrresprodped($cdfilial, 
                                                                                                    $nrlanctonf, 
                                                                                                    $itemP['NRSEQITPROD']);
                                        $nrresprodped = $nrresprodped[0]['NRRESPRODPED'];
                                        
                                        // Insere reserva caso não exista
                                        $this->ExpedicaoService->insereReservaItemprod($cdfilial, 
                                                                                        $nrlanctonf, 
                                                                                        $itemP['NRSEQITPROD'],
                                                                                        $nrresprodped,
                                                                                        $itemP['CDPRODUTO'],
                                                                                        $this->resultValue($itemP['CDALMOXARIFE']),
                                                                                        $this->resultValue($itemP['CDLOCALESTOQ']),
                                                                                        $this->resultValue($coleta['NRLOTEESTQ']), 
                                                                                        $this->resultValue($coleta['NRSUBLOTE']),
                                                                                        $qtSeparada,
                                                                                        '');
                                                                                        
                                    }
                                }
                                
                            // FIM SE QUANTIDADE A EXPEDIR FOR MAIOR QUE A QUE FALTA
                            }else{
                            // INICIO SE QUANTIDADE A EXPEDIR FOR MENOR QUE A QUANTIDADE QUE FALTA
                            
                                $qtSeparada = $itemP['QTSEPARADA'] + $qtresprodped;
                                $qtFaltaSepIt = $itemP['QTEXPEDIR'] - $qtSeparada;
                                $boAcabou = true;
                            
                                // boAcabou := True;
                                
                                if($qtFaltaSepIt > 0){
                                    $idstatusreser = 'PA';
                                }else{
                                    $idstatusreser = 'LQ';
                                }
            
                                $this->ExpedicaoService->updateStItemproduto($cdfilial, 
                                                                                $nrlanctonf, 
                                                                                $itemP['NRSEQITPROD'],
                                                                                $qtSeparada,
                                                                                $idstatusreser);
                                // Busca RESERITEMPROD
                                $reservaItemprod = $this->ExpedicaoService->getReservaItemprod($cdfilial, 
                                                                                                $nrlanctonf, 
                                                                                                $itemP['NRSEQITPROD'], 
                                                                                                $itemP['CDPRODUTO'], 
                                                                                                $this->resultValue($itemP['CDALMOXARIFE']), 
                                                                                                $this->resultValue($itemP['CDLOCALESTOQ']), 
                                                                                                $this->resultValue($coleta['NRLOTEESTQ']), 
                                                                                                $this->resultValue($coleta['NRSUBLOTE']));
            
                                if(count($reservaItemprod)){
                                    // Updata reserva caso já exista
                                    $this->ExpedicaoService->updateReservaItemprod($qtSeparada,
                                                                                    $cdfilial, 
                                                                                    $nrlanctonf, 
                                                                                    $itemP['NRSEQITPROD'],
                                                                                    $reservaItemprod[0]['NRRESPRODPED']);
                                }else{
                                    // Gera sequencial da coleta 
                                    $nrresprodped = $this->ExpedicaoService->getMaxNrresprodped($cdfilial, 
                                                                                                $nrlanctonf, 
                                                                                                $itemP['NRSEQITPROD']);
                                    $nrresprodped = $nrresprodped[0]['NRRESPRODPED'];
                                    
            
                                    // Insere reserva caso não exista
                                    $this->ExpedicaoService->insereReservaItemprod($cdfilial, 
                                                                                    $nrlanctonf, 
                                                                                    $itemP['NRSEQITPROD'],
                                                                                    $nrresprodped,
                                                                                    $itemP['CDPRODUTO'],
                                                                                    $this->resultValue($itemP['CDALMOXARIFE']),
                                                                                    $this->resultValue($itemP['CDLOCALESTOQ']),
                                                                                    $this->resultValue($coleta['NRLOTEESTQ']), 
                                                                                    $this->resultValue($coleta['NRSUBLOTE']),
                                                                                    $qtresprodped,
                                                                                    '');
            
                                }
                                
                                if($boAcabou){
                                    break;
                                }
                                
                            // FIM SE QUANTIDADE A EXPEDIR FOR MENOR QUE A QUANTIDADE QUE FALTA
                            }
                            
                            $rownum++;
                            
                        // FIM FOREACH DE ITEMPRODUTO
                        }
                        
                    // FIM SE TEM PRODUTO
                    }else{
                        
                        $maxItemprod = $this->ExpedicaoService->getMaxItemprod($cdfilial, $nrlanctonf, $cdproduto);
                        
                        $this->ExpedicaoService->updateStItemproduto($cdfilial, 
                                                                    $nrlanctonf, 
                                                                    $maxItemprod[0]['NRSEQITPROD'],
                                                                    $maxItemprod[0]['QTSEPARADA'] + $qtresprodped,
                                                                    'LQ');
                                                                    
                        // Busca RESERITEMPROD
                        $reservaItemprod = $this->ExpedicaoService->getReservaItemprod($cdfilial, 
                                                                                        $nrlanctonf, 
                                                                                        $maxItemprod[0]['NRSEQITPROD'], 
                                                                                        $maxItemprod[0]['CDPRODUTO'], 
                                                                                        $this->resultValue($maxItemprod[0]['CDALMOXARIFE']), 
                                                                                        $this->resultValue($maxItemprod[0]['CDLOCALESTOQ']), 
                                                                                        $this->resultValue($coleta['NRLOTEESTQ']), 
                                                                                        $this->resultValue($coleta['NRSUBLOTE']));
            
                        if(count($reservaItemprod)){
                            // Updata reserva caso já exista
                            $this->ExpedicaoService->updateReservaItemprodSoma($qtresprodped,
                                                                                $cdfilial, 
                                                                                $nrlanctonf, 
                                                                                $maxItemprod[0]['NRSEQITPROD'],
                                                                                $reservaItemprod[0]['NRRESPRODPED']);
                        }else{
                            // Gera sequencial da coleta 
                            $nrresprodped = $this->ExpedicaoService->getMaxNrresprodped($cdfilial, 
                                                                                        $nrlanctonf, 
                                                                                        $maxItemprod[0]['NRSEQITPROD']);
                            $nrresprodped = $nrresprodped[0]['NRRESPRODPED'];
                            
                            // Insere reserva caso não exista
                            $this->ExpedicaoService->insereReservaItemprod($cdfilial, 
                                                                            $nrlanctonf, 
                                                                            $maxItemprod[0]['NRSEQITPROD'],
                                                                            $nrresprodped,
                                                                            $maxItemprod[0]['CDPRODUTO'],
                                                                            $this->resultValue($maxItemprod[0]['CDALMOXARIFE']),
                                                                            $this->resultValue($maxItemprod[0]['CDLOCALESTOQ']),
                                                                            $this->resultValue($coleta['NRLOTEESTQ']), 
                                                                            $this->resultValue($coleta['NRSUBLOTE']),
                                                                            $qtresprodped,
                                                                            '');
                        }
                        
                        $qtresprodped = 0;
                    }
                    
                // FIM DO FOREACH DA COLETA
                }
                
            }
         
        // Valida situação do volume de acordo com coleta realizada.
        $status = $this->validaSitVolume($cdfilial, $nrlanctonf, $nrseqcontainer);
        
        // Valida se existe coleta para a pre NF selecionada, se existe updata a NF pra "Em andamento."
        $this->validaSitPrenf($cdfilial, $nrlanctonf);
        
        $data = array('STATUS' => $status);
        $dataSet = new DataSet ("data", $data);
        $response->addDataSet($dataSet);
        
        /* Commit */
        $this->entityManager->getConnection()->commit();

        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
            
            /* rollback */
            $this->entityManager->rollback();
        }
    }
    
    /*public function updateQtColetadaVol($request, $response){ 
        $row = $request->getRow();
        $cdfilial           = $row[0]['CDFILIAL'];
        $nrlanctonf         = $row[0]['NRLANCTONF'];
        $nrseqcontainer     = $row[0]['NRSEQCONTAINER'];
        
        try {
            $qtcol = $this->ExpedicaoService->getQtColetadaVol($cdfilial, $nrlanctonf, $nrseqcontainer);
            $qtcol = $qtcol[0]['TOTQTCOLETADA'];
            
            // Atualizar a quantidade coletada no volume 
            $this->ExpedicaoService->updateQtColetadaVol($qtcol, $cdfilial, $nrlanctonf, $nrseqcontainer);
            
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }*/
    
    public function updateStItemproduto($request, $response){
        try {
            $row = $request->getRow();
            $idstatusreser      = $row[0]['IDSTATUSRESER'];
            $qtSeparada         = '';
            $cdfilial           = $row[0]['CDFILIAL'];
            $nrlanctonf         = $row[0]['NRLANCTONF'];
            $nrseqitprod        = $row[0]['NRSEQITPROD'];
            
            $this->ExpedicaoService->updateStItemproduto($idstatusreser, $qtSeparada, $cdfilial, $nrlanctonf, $nrseqitprod);
            
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deleteColeta($request, $response){
        try {
            $row = $request->getRow();
            $cdfilial           = $row[0]['CDFILIAL'];
            $nrlanctonf         = $row[0]['NRLANCTONF'];
            $cdproduto          = $row[0]['CDPRODUTO'];
            $nrseqcontainer     = $row[0]['NRSEQCONTAINER'];
            
            $this->ExpedicaoService->deleteColeta($cdfilial, $nrlanctonf, $cdproduto, $nrseqcontainer);
            
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function validaExclusaoVolume($request, $response){
        $dataSet = $request->getDataSet();
        $volumes = $dataSet->getRows();
        try {
            
            $lotes = '';
            foreach ($volumes as $volume) {
                $cdfilial       = $volume['CDFILIAL'];
                $nrlanctonf     = $volume['NRLANCTONF'];
                $nrseqcontainer = $volume['NRSEQCONTAINER'];
                
                $coleta = $this->ExpedicaoService->getColetaPorVolume($cdfilial, $nrlanctonf, $nrseqcontainer);
                
                if(count($coleta) > 0){
                    $lotes = $lotes. str_pad($volume['NRSEQCONTAINER'], 4, "0", STR_PAD_LEFT)." - ".$volume['NMCONTAINER']."<br>";
                }
            }
            
            if($lotes){
                throw new \Exception("Impossível excluir, os seguintes volumes possuem coleta:<br><br>".$lotes);
            }
            
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    } 
    
    public function concluirVolume($request, $response){
        try {
            $volumes = $request->getRow();
            
            foreach ($volumes as $volume) {

                $cdfilial           = $volume['CDFILIAL'];
                $nrlanctonf         = $volume['NRLANCTONF'];
                $nrseqcontainer     = $volume['NRSEQCONTAINER'];

                // BUSCA COLETAS POR VOLUME E VERIFICA SE ALGUMA ESTA NO STATUS DE 'E'
                $coletas = $this->ExpedicaoService->getColetaExcedida($cdfilial, $nrlanctonf, $nrseqcontainer);
                if(count($coletas)){
                    /*foreach ($coletas as $coleta) {
                       
                        // Se tiver alguma coleta com quantidade excedida alem da % permitida, retorna para usuário e não conclui volume.
                        if($coleta['IDCOLETA'] === 'E'){
                            throw new \Exception("Impossível concluir volume(s), existe(m) coleta(s) excedida(s).");
                        }
                        
                    }*/
                    
                    // (Nova rotina - Guilherme) Conclui volume sem validação de excedido
                    $this->ExpedicaoService->updateStVolumeConcluido($cdfilial, $nrlanctonf, $nrseqcontainer, 'CL');
                }else{
                    throw new \Exception("Volume vazio selecionado, exclua o volume ou realize uma coleta.");
                }
            }
            
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function encerrarColeta($request, $response){
        
        try {
            $row = $request->getRow();
            
            $cdfilial           = $row[0]['CDFILIAL'];
            $nrlanctonf         = $row[0]['NRLANCTONF'];
            
            // BUSCA VOLUMES DA EXPEDIÇÃO
            $volumes = $this->ExpedicaoService->getVolumesExp($cdfilial, $nrlanctonf);
            if($volumes){
                foreach ($volumes as $volume) {

                    // Se tiver algum volume não concluído, retorna mensagem para usuário.
                    if($volume['IDSTATUSCOL'] <> 'CL'){
                        throw new \Exception("Impossível encerrar coleta, existe(m) volume(s) não concluído(s).");
                    }
                }
            }else{
                throw new \Exception("Não existem volumes para essa expedição.");
            }
            
            // Updata STATUS da notafiscal se todos os volumes tiverem concluídos.
            $this->ExpedicaoService->updateStNfColetada($cdfilial, $nrlanctonf, 'E');
            
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getSeparacao($request, $response){
        $row = $request->getRow();
        
        $data = $this->ExpedicaoService->getSeparacao($row[0]['PARAMS']);
        $dataSet = new DataSet ("data", $data);
        $response->addDataSet($dataSet);
        
    }

}