<?php
namespace Teknisa\PED\Modules\Pedidos\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\PED\Modules\Pedidos\Factories\ped02300PrecoBaseFactory;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Environment;

class ped02300PrecoBaseService{

	protected $entityManager;
	protected $environment;
	protected $newCode;
	protected $PrecoBaseFactory;

	public function __construct(EntityManager $entityManager, NewCode $newCode, Environment $environment, ped02300PrecoBaseFactory $PrecoBaseFactory){
		$this->entityManager           = $entityManager;
		$this->environment				= $environment;
		$this->newCode					= $newCode;
		$this->ped02300PrecoBaseFactory = $PrecoBaseFactory;
	}

	public function validaIntervalo($row,$nrorg){
	    return $this->ped02300PrecoBaseFactory->setValidadeIntervalo($row[0]['CDPRECOBASE'],$row[0]['CDPRODUTO'],$row[0]['QTCONVINIPRB'],$row[0]['QTCONVFINPRB'],$nrorg);
	}

	public static function formataData($data){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
	        $data = DateUtil::getDataDeString($data);
	        $data = DateUtil::truncateData($data);

        return $data;
    }

	public function insereIten($rows){
		$connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
			// Percorre Tabelas de Preço a serem Replicadas
            for ($i = 0; $i < count($rows[0][1]); $i++) {
            	$tabela = $rows[0][1][$i];

            	// Busca Tabela de Preço
            	$precobase = $this->ped02300PrecoBaseFactory->buscaPrecobase($tabela["NRORG"], $tabela["CDPRECOBASE"]);

				// Percorre Produtos e Preços a serem Replicados
				for ($e = 0; $e < count($rows[0][0]); $e++) {
				    $item = $rows[0][0][$e];

				    $item_old = $this->ped02300PrecoBaseFactory->buscaItemprebase($item["NRORG"], $tabela["CDPRECOBASE"], $item["CDPRODUTO"]);
				
				 if($item_old){
				    	$this->entityManager->remove($item_old);
				    	$this->entityManager->flush();
				    }

				    // Insere Item Preco Base
				    $itemprebase = $this->ped02300PrecoBaseFactory->insereItemprebase(
				        $item["NRORG"], $tabela["CDPRECOBASE"], $item["CDPRODUTO"],
				        $this->formataData($item["DTINIVALIPB"]), $this->formataData($item["DTFIMVALIPB"]),
				        $item["VRITEPREBASE"], $this->formataData($item["DTHRATZIPB"]), $item["CDOPERATZIPB"]
				    );
				   
				    $this->entityManager->persist($itemprebase);
				    $this->entityManager->flush();

				    // Verifica Utilozação de Preço Promocinal
				    if($precobase->getIdultprecpromo() == 'S'){

				    	// Busca Promoçao(ões)
				    	$promoprbase = $this->ped02300PrecoBaseFactory->buscaPromoprbase(
				    		$item["NRORG"], $item["CDPRECOBASE"], $item["CDPRODUTO"]
				    	);

				    	// Verifica Existência de Promoção(ões)
				    	if($promoprbase){
				    		// Percorre Promoção(ões) Existênte(s)
				    		foreach ($promoprbase as $promopr) {

				    			// Insere Nova Promoção do Produto da Tabela Replicada

				    			$promoprbase_new = $this->ped02300PrecoBaseFactory->inserePromoprbase(
							        $item["NRORG"], $tabela["CDPRECOBASE"], $item["CDPRODUTO"],
							        $this->formataData($item["DTINIVALIPB"]), $promopr->getIddiaspromo(), $promopr->getDtinipromo(),
							        $this->formataData($promopr->getDtfinpromo()), $promopr->getIdtipopromo(), $promopr->getVrperbasepromo(),
							        $promopr->getVrtotalpromo(), $promopr->getIdstatuspromo()
							    );

							    $this->entityManager->persist($promoprbase_new);
				    		}
				    	}
					}else{

						// Busca Preço(s) Opcional(is)
				    	$opcioprbase = $this->ped02300PrecoBaseFactory->buscaOpcioprbase(
				    		$item["NRORG"], $item["CDPRECOBASE"], $item["CDPRODUTO"]
				    	);

				    	// Verifica Existência de Preço(s) Opcional(is)
				    	if($opcioprbase){
				    		// Percorre Preço(s) Opcional(is) Existênte(s)
				    		foreach ($opcioprbase as $opciopr) {

				    			if($opciopr_fator = $this->ped02300PrecoBaseFactory->buscaOpcioprbaseFator(
				    					$item["NRORG"], $tabela["CDPRECOBASE"], $item["CDPRODUTO"],
				    					$opciopr->getCdfatpreco())
				    				){
				    				$this->entityManager->remove($opciopr_fator);
				    				$this->entityManager->flush();

				    			}

				    			//Gerando data de inclusao
				    			$dtinclusao = date("d/m/Y 00:00:00");
				    			$dtinclusao = $this->formataData($dtinclusao);

				    			//Gera novo codigo para o campo NRSEQOPCIOPRBASE para o Preço Replicado
				    			$novoCodigo = $this->newCode->getCodeByTableName("OPCIOPRBASE", $item["NRORG"]);

				    			// Insere Novo Preço Opcional do Produto da Tabela Replicada
				    			$opcioprbase_new = $this->ped02300PrecoBaseFactory->insereOpcioprbase(
							        $item["NRORG"], $tabela["CDPRECOBASE"], $item["CDPRODUTO"], $opciopr->getCdfatpreco(),
							        $opciopr->getVropcioprbase(), $novoCodigo, $dtinclusao
							    );

							    $this->entityManager->persist($opcioprbase_new);
				    		}
				    	}
					}

					// Busca Convidado(s)
			    	$convidado = $this->ped02300PrecoBaseFactory->buscaPrbaseconvidaore(
			    		$item["NRORG"], $item["CDPRODUTO"], $item["CDPRECOBASE"]
			    	);

			    	// Verifica Existência de Convidado(s)
			    	if($convidado){
			    		// Percorre Convidado(s) Existênte(s)
			    		foreach ($convidado as $conv) {

			    			// Insere Novo Convidado da Tabela Replicada
			    			$convidado_new = $this->ped02300PrecoBaseFactory->inserePrbaseconvidaore(
						        $item["NRORG"], $tabela["CDPRECOBASE"], $item["CDPRODUTO"], $this->formataData($item["DTINIVALIPB"]),
						        $conv->getQtconviniprb(), $conv->getQtconvfinprb(), $conv->getVrprebaseconv(), $conv->getQthrduracaopb()
						    );

						    $this->entityManager->persist($convidado_new);
			    		}
			    	}
				}
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
	}

	public function reajusteIten($rows){
		$connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
        	for($i = 0 ; $i < count($rows[0][0]); $i++){
        		$dadosItem = $rows[0][0][$i];

        		//Busca Item
        		$item = $this->ped02300PrecoBaseFactory->buscaItemprebaseReajuste(
        			$dadosItem["NRORG"], $dadosItem["CDPRECOBASE"], $dadosItem["CDPRODUTO"]
        		);


        		foreach($item as $itemNovo){
        			$novoValor;

        			//Calcula acrescimo ou decrescimo
        			if($rows[0][3] == "A"){
        				$novoValor = $itemNovo->getVriteprebase() + (($itemNovo->getVriteprebase() * $rows[0][1]) / 100);
        			} else {
        				$novoValor = $itemNovo->getVriteprebase() - (($itemNovo->getVriteprebase() * $rows[0][1]) / 100);
        			}

        			//Sete os novos valores aos campos
        			$itemNovo->setVriteprebase($novoValor);
        			$itemNovo->setDtfimvalipb($this->formataData($rows[0][2]));

        			$this->entityManager->persist($itemNovo);

        		}

        		//Busca a tabela do item selecionado
        		$precobase = $this->ped02300PrecoBaseFactory->buscaPrecobase(
        			$dadosItem["NRORG"], $dadosItem["CDPRECOBASE"]
        		);

        		//Se a tabela aceitar preco promocional
        		if($precobase->getIdultprecpromo() == 'S'){

        			//Busca Promoção(ões)
        			$promoprbase = $this->ped02300PrecoBaseFactory->buscaPromoprbase(
				    		$dadosItem["NRORG"], $dadosItem["CDPRECOBASE"], $dadosItem["CDPRODUTO"]
				    );

				    //Se tem promoção(ões)
				    if($promoprbase){

				    	foreach($promoprbase as $promo){

				    		//Excluindo promoções
        					$this->entityManager->remove($promo);
        					$this->entityManager->flush();

				    	}
				    }

        		} else {

        			$opcioprbase = $this->ped02300PrecoBaseFactory->buscaOpcioprbase(
        				$dadosItem["NRORG"], $dadosItem["CDPRECOBASE"], $dadosItem["CDPRODUTO"]
        			);

        			//Se tem preço(s) opicinal(is)
				    if($opcioprbase){

				    	foreach($opcioprbase as $opiciopr){
        					$novoValor;

		        			//Calcula acrescimo ou decrescimo
		        			if($rows[0][3] == "A"){
		        				$novoValor = $opiciopr->getVropcioprbase() + (($opiciopr->getVropcioprbase() * $rows[0][1]) / 100);
		        			} else {
		        				$novoValor = $opiciopr->getVropcioprbase() - (($opiciopr->getVropcioprbase() * $rows[0][1]) / 100);
		        			}

		        			$opiciopr->setVropcioprbase($novoValor);

        					$this->entityManager->persist($opiciopr);


				    	}
				    }
        		}

        		// Busca Convidado(s)
			    $convidado = $this->ped02300PrecoBaseFactory->buscaPrbaseconvidaore(
			    	$dadosItem["NRORG"], $dadosItem["CDPRODUTO"], $dadosItem["CDPRECOBASE"]
			    );

			    if($convidado){
			    	// Percorre Convidado(s) Existênte(s)
		    		foreach ($convidado as $conv) {
					    $novoValor;

	        			//Calcula acrescimo ou decrescimo
	        			if($rows[0][3] == "A"){
	        				$novoValor = $conv->getVrprebaseconv() + (($conv->getVrprebaseconv() * $rows[0][1]) / 100);
	        			} else {
	        				$novoValor = $conv->getVrprebaseconv() - (($conv->getVrprebaseconv() * $rows[0][1]) / 100);
	        			}

		        		$conv->setVrprebaseconv($novoValor);
					    $this->entityManager->persist($conv);
		    		}
			    }
        	}
        	$this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
        	$connection->rollBack();
            throw $e;
        }
	}
	
	public function updateDescMaximo($row, $cdprecobase, $sgestadoAntigo, $cdprodutoAntigo){
	    return $this->ped02300PrecoBaseFactory->updateDescMaximo($row, $cdprecobase, $sgestadoAntigo, $cdprodutoAntigo);
	}
	
	public function salvarCadAutoFator($row, $response){
	   $produtos =  $this->ped02300PrecoBaseFactory->buscaProdutos($row);
	    
	   foreach($produtos as $produto){
	   	    $fatorProduto = $this->ped02300PrecoBaseFactory->selectProdfator($produto['CDPRODUTO'], $row['CDFATPRECO']);
	   	    if(empty($fatorProduto)){
	   	    	$this->ped02300PrecoBaseFactory->insereProdFator($row['CDFATPRECO'], $produto['CDPRODUTO'], $row['VRFATPRECO'], $response);
	   	    }
	   	    else{
	   	    	$this->ped02300PrecoBaseFactory->updateProdFator($row['CDFATPRECO'], $produto['CDPRODUTO'], $row['VRFATPRECO'], $response);
	   	    }
	   }
	    
	}

}

?>